#include <game.h>

lieu::lieu() : base("lieu")
{
	listStatic = new std::vector<staticMesh*>;
	listAction = new std::vector<actionMesh*>;
}

lieu::~lieu()
{
	delete listAction;
	delete listStatic;
}

int lieu::init(int i,coeur* c,IXMLReader* xmlLieu)
{
	base::init(i,c);
	log(stringc("Lieu xml : ") + stringc(i) + " loading ...");
	staticMesh* mesh = 0;
	ILightSceneNode* light = 0;
	while(xmlLieu && xmlLieu->read())
	{
		if (xmlLieu->getNodeType()==EXN_ELEMENT)
		{
			if (core::stringw("lieu") == xmlLieu->getNodeName())
			{
				listStatic->reserve(xmlLieu->getAttributeValueAsInt(L"nbrstatic"));
				listAction->reserve(xmlLieu->getAttributeValueAsInt(L"nbraction"));
			}
			else if (core::stringw("light") == xmlLieu->getNodeName())
			{
				light = mycore->addLight(xmlLieu->getAttributeValueAsInt(L"radius"));
				log( stringc("New light : radius: ") + xmlLieu->getAttributeValue(L"radius") );
			}
			else if (core::stringw("static") == xmlLieu->getNodeName())
			{
				mesh=addStaticMesh(
					stringc(xmlLieu->getAttributeValue(L"model")),
					xmlLieu->getAttributeValueAsInt(L"id")
				);
				log( stringc("New static : id:") + xmlLieu->getAttributeValue(L"id") + ", model: " + xmlLieu->getAttributeValue(L"model"));
			}
			else if (core::stringw("observation") == xmlLieu->getNodeName())
			{
				mesh=addObservationMesh(
					stringc(xmlLieu->getAttributeValue(L"model")),
					xmlLieu->getAttributeValueAsInt(L"id"),
					xmlLieu->getAttributeValueAsInt(L"combiId")
				);
				log( stringc("New observation : id:") + xmlLieu->getAttributeValue(L"id") + ", model: " + xmlLieu->getAttributeValue(L"model") + ", combiId: " + xmlLieu->getAttributeValue(L"combiId"));
			}
			else if (core::stringw("prendre") == xmlLieu->getNodeName())
			{
				mesh=addPrendreMesh(
					stringc(xmlLieu->getAttributeValue(L"model")),
					xmlLieu->getAttributeValueAsInt(L"id"),
					xmlLieu->getAttributeValueAsInt(L"combiId"),
					xmlLieu->getAttributeValueAsInt(L"itemId")
				);
				log( stringc("New prendre : id:") + xmlLieu->getAttributeValue(L"id") + ", model: " + xmlLieu->getAttributeValue(L"model") + ", combiId: " + xmlLieu->getAttributeValue(L"combiId") + ", itemId: " + xmlLieu->getAttributeValue(L"itemId"));
			}
			else if (core::stringw("animate") == xmlLieu->getNodeName())
			{
				mesh=addAnimateMesh(
					stringc(xmlLieu->getAttributeValue(L"model")),
					xmlLieu->getAttributeValueAsInt(L"id"),
					xmlLieu->getAttributeValueAsInt(L"combiId"),
					xmlLieu->getAttributeValueAsInt(L"animId"),
					xmlLieu->getAttributeValueAsInt(L"begin"),
					xmlLieu->getAttributeValueAsInt(L"end")
				);
				log( stringc("New animated : id:") + xmlLieu->getAttributeValue(L"id") + ", model: " + xmlLieu->getAttributeValue(L"model") + ", combiId: " + xmlLieu->getAttributeValue(L"combiId") + ", animId: " + xmlLieu->getAttributeValue(L"animId") + ", begin: " + xmlLieu->getAttributeValue(L"begin")  + ", end: " + xmlLieu->getAttributeValue(L"end"));
			}
			else if (core::stringw("position") == xmlLieu->getNodeName())
			{
				ISceneNode* node = 0;
				if (mesh)
					node = mesh->getNode();
				else if (light)
					node = light;
				else
				{
					log(stringc("	Bad format : Position comand bad placed"));
					continue;
				}
				
				node->setPosition(vector3df(
					xmlLieu->getAttributeValueAsFloat(L"x"),
					xmlLieu->getAttributeValueAsFloat(L"y"),
					xmlLieu->getAttributeValueAsFloat(L"z")
				));
				
				log( stringc("	Position : (") + xmlLieu->getAttributeValue(L"x") + "," + xmlLieu->getAttributeValue(L"y") + "," + xmlLieu->getAttributeValue(L"z") + ")");
			}
			else if (core::stringw("rotation") == xmlLieu->getNodeName())
			{
				mesh->getNode()->setRotation(vector3df(
					xmlLieu->getAttributeValueAsFloat(L"x"),
					xmlLieu->getAttributeValueAsFloat(L"y"),
					xmlLieu->getAttributeValueAsFloat(L"z")
				));
				log( stringc("	Rotation : (") + xmlLieu->getAttributeValue(L"x") + "," + xmlLieu->getAttributeValue(L"y") + "," + xmlLieu->getAttributeValue(L"z") + ")");
			}
			else if (core::stringw("scale") == xmlLieu->getNodeName())
			{
				mesh->getNode()->setScale(vector3df(
					xmlLieu->getAttributeValueAsFloat(L"x"),
					xmlLieu->getAttributeValueAsFloat(L"y"),
					xmlLieu->getAttributeValueAsFloat(L"z")
				));
				log( stringc("	Scale : (") + xmlLieu->getAttributeValue(L"x") + "," + xmlLieu->getAttributeValue(L"y") + "," + xmlLieu->getAttributeValue(L"z") + ")");
			}
			else if (core::stringw("color") == xmlLieu->getNodeName())
			{
				light->getLightData().DiffuseColor = SColorf(
					xmlLieu->getAttributeValueAsFloat(L"red"),
					xmlLieu->getAttributeValueAsFloat(L"green"),
					xmlLieu->getAttributeValueAsFloat(L"blue"),
					xmlLieu->getAttributeValueAsFloat(L"alpha")
				);
				log( stringc("	Color : (") + xmlLieu->getAttributeValue(L"red") + "," + xmlLieu->getAttributeValue(L"green") + "," + xmlLieu->getAttributeValue(L"blue") + "," + xmlLieu->getAttributeValue(L"alpha") + ")");
			}
			else if (core::stringw("skybox") == xmlLieu->getNodeName())
			{
				mycore->setSkyBox(
					xmlLieu->getAttributeValue(L"top"),
					xmlLieu->getAttributeValue(L"bottom"),
					xmlLieu->getAttributeValue(L"left"),
					xmlLieu->getAttributeValue(L"right"),
					xmlLieu->getAttributeValue(L"front"),
					xmlLieu->getAttributeValue(L"back")
				);
				log( stringc("	Skybox"));//TODO
			}
			else if (core::stringw("texture") == xmlLieu->getNodeName())
			{
				mesh->getNode()->setMaterialTexture(xmlLieu->getAttributeValueAsInt(L"id"),mycore->getTexture(xmlLieu->getAttributeValue(L"path")));
				log( stringc("	Texture"));//TODO
			}
		}
		if (xmlLieu->getNodeType()==EXN_ELEMENT_END)
		{
			if ((core::stringw("static") == xmlLieu->getNodeName())||(core::stringw("observation") == xmlLieu->getNodeName())||(core::stringw("prendre") == xmlLieu->getNodeName()))
				mesh=0;
			else if (core::stringw("light") == xmlLieu->getNodeName())
				light=0;
		}
	}


	return 1;
}

int lieu::update()
{
	return 1;
}

int lieu::close()
{
	log("Closing each staticMesh");
	while (! listStatic->empty())
	{
		log(stringc("Deleting staticMesh ") + stringc(listStatic->back()->getId()));
		delete listStatic->back();
		listStatic->pop_back();
	}
	log("Closing each actionMesh");
	while (!listAction->empty())
	{
		log(stringc("Deleting actionMesh ") + stringc(listAction->back()->getId()));
		delete listAction->back();
		listAction->pop_back();
	}
	base::close();
	return 1;
}

staticMesh* lieu::addStaticMesh(stringc path,int i)
{
	staticMesh *m = new staticMesh;
	m->init(i,mycore->addMesh(path));
	m->getNode()->setID(i);
	listStatic->push_back(m);
	log("staticMesh added");
	return m;
}

observationMesh* lieu::addObservationMesh(stringc path, int i, int c)
{
	observationMesh *m = new observationMesh;
	m->init(i,mycore->addMesh(path),c);
	m->getNode()->setID(i);
	listAction->push_back(m);
	log("observationMesh added");
	return m;
}

prendreMesh* lieu::addPrendreMesh(stringc path, int i, int c, int ite)
{
	prendreMesh *m = new prendreMesh;
	m->init(i,mycore->addMesh(path),c, ite);
	m->getNode()->setID(i);
	listAction->push_back(m);
	log("prendreMesh added");
	return m;
}

animateMesh* lieu::addAnimateMesh(stringc path, int i, int c, int ite, int b, int e)
{
	animateMesh *m = new animateMesh;
	m->init(i,mycore->addMesh(path),c, ite, b ,e);
	m->getNode()->setID(i);
	listAction->push_back(m);
	log("animateMesh added");
	return m;
}

staticMesh* lieu::getStaticMeshFromId(int id)
{
	for (unsigned int i=0;i<listStatic->size();i++)
		if (listStatic->at(i)->getId() == id)
		{
			return listStatic->at(i);
		}
	return 0;
}

actionMesh* lieu::getActionMesh(int id)
{
	for (unsigned int i=0;i<listAction->size();i++)
		if (listAction->at(i)->getId() == id)
		{
			return listAction->at(i);
		}
	return 0;
}

actionMesh* lieu::getActionMesh(ISceneNode* node)
{
	for (unsigned int i=0;i<listAction->size();i++)
		if (listAction->at(i)->getNode() == node)
		{
			return listAction->at(i);
		}
	return 0;
}

bool lieu::OnEvent(const SEvent& event)
{
	return false;
}

