/* This file is part of The_Merry_Turnip
 * 
 * The_Merry_Turnip - TODO: Description
 * Copyright © DAVY Guillaume 2009 <davyg2@gmail.com>
 * 
 * The_Merry_Turnip is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The_Merry_Turnip is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*!	\file pyInt.cpp
	\brief Implement pyInt class
*/

#include <game.h>
#include <iostream>

/*!
	\class pyInt
	It will create the python embedded system and make it accesible for other class
	in order to launch instruction, script and to get the output
*/

extern "C" void initpyIrr();
extern "C" void initpyMerry();

//! Construct the pyInt class, call base::base()
pyInt::pyInt() : base("pyInt")
{
	
}

//! Destruct the pyInt class
pyInt::~pyInt()
{
	
}

/** Call base::init() and initialize python
	\param c Pointer to the core
	\return 0 if succesful, -1 instead*/
int pyInt::init(coeur* c)
{	
	//Init the base(no need the return value, which is the logId)
	base::init(0,c);
	
	//Init python
	char modIrr[] = "pyIrr";
	char modMerry[] = "pyMerry";
	PyImport_AppendInittab(modIrr, initpyIrr);
	PyImport_AppendInittab(modMerry, initpyMerry);
	Py_Initialize();
	main_module = import("__main__");
	dict = main_module.attr("__dict__");
	exec("from pyIrr import *");
	exec("from pyMerry import *");
	addPyEnv("core",object(ptr(c)));

	return 0;
}

/** Does nothing, apart from slowing the game
	\return always 0*/
int pyInt::update()
{
	return 0;
}

/** Quit python and call base::close()
	\return base::close()*/
int pyInt::close()
{
	Py_Finalize();
	return base::close();//Close the base
}

/** This method is called when an event has to be treated, but this class
	do not wait event so this function does nothing
	\param event The event to parse
	\return Always false*/
bool pyInt::OnEvent(const SEvent& event)
{
	return false;
}

/** This method run a python command
	\param str command to run
	\return the result of the command*/
stringw pyInt::eval(stringw str)
{
	object r;
	try
	{
		r = boost::python::exec(stringc(str).c_str(),dict,dict);
	}
	catch (...)
	{
		PyErr_Print();
		log(stringc("Error in the command ") + str);
		return stringw(L"Error in the command : ") + str + "\n";
	}
	//addPyEnv("my__result",r);
	//boost::python::exec("print my__result",dict,dict);
	log(stringc("Run(exec) succesfully the command ") + str);
	return stringc("");
}

/** This method run a python script
	\param str script to run
	\return 0 if succsful, -1 instead*/
int pyInt::exec(stringw str)
{
	try
	{
		boost::python::exec(stringc(str).c_str(),dict,dict);
	}
	catch (...)
	{
		log(stringc("Error in the script") + str);
		return -1;
	}
	log(stringc("Run succesfully the script\n###################\n") + str + "\n###################\n");
	return 0;
}

/** This method add a variable to the python envirronement
	\param s Name of the variable
	\param o The object to put in the variable
	\return 0*/
int pyInt::addPyEnv(stringc s,object o)
{
	main_module.attr(s.c_str()) = o;
	return 0;
}


